import React, { useState } from "react";
import { Table, OverlayTrigger, Popover } from "react-bootstrap";
import { LinkContainer } from "react-router-bootstrap";
import { FaShoppingBasket, FaMoneyBillAlt } from "react-icons/fa";
import { GiPayMoney, GiReceiveMoney } from "react-icons/gi";
import { HiClipboardCopy, HiClipboardList } from "react-icons/hi";
import Fetcher from "./Fetcher";
import { printMoney, printDate } from "./util";

function icon(transaction) {
  switch (transaction.type) {
    case "purchase":
      return <FaShoppingBasket />;
    case "topup":
      if (transaction.total < 0) {
        return <GiReceiveMoney />;
      }
      return <GiPayMoney />;
    case "order":
      return <HiClipboardList />;
    case "refund":
      return <HiClipboardCopy />;
    default:
      return <FaMoneyBillAlt />;
  }
}

function transactionOverlay(transaction) {
  let title;
  let content;
  switch (transaction.type) {
    case "purchase":
      title = "compra";
      content = transaction.purchase.map((p) => p.product.name).join(",") + ".";
      break;
    case "topup":
      if (transaction.total < 0) {
        title = "devolución";
      } else {
        title = "recarga";
      }
      content = transaction.topup.comment;
      break;
    case "order":
      title = "pedido de " + transaction.order.name;
      content = transaction.order_purchase.map((p) => {
        return (
          <div key={"O" + transaction.ID + "-" + p.order_product.ID}>
            {p.order_product.product.name + ": " + p.amount}
            <br />
          </div>
        );
      });
      break;
    case "refund":
      title = "devolución de " + transaction.refund.name;
      break;
    default:
      title = "transacción";
  }
  return (
    <Popover>
      <Popover.Title>{title}</Popover.Title>
      {content && <Popover.Content>{content}</Popover.Content>}
    </Popover>
  );
}

function TransactionList() {
  const [transactions, setTransactions] = useState([]);

  // TODO: useEffect to disable fetcher...

  const entries = transactions.map((transaction) => {
    const colorClass = transaction.total < 0 ? "table-danger" : "table-success";
    return (
      <OverlayTrigger
        overlay={transactionOverlay(transaction)}
        key={transaction.ID}
      >
        <LinkContainer to={"/transaction/" + transaction.ID}>
          <tr className={colorClass}>
            <td>{icon(transaction)}</td>
            <td>{printDate(transaction.date)}</td>
            <td>{printMoney(transaction.total) + " €"}</td>
          </tr>
        </LinkContainer>
      </OverlayTrigger>
    );
  });

  return (
    <Fetcher url="/api/transaction/mine" onFetch={setTransactions}>
      <Table className="text-center">
        <thead>
          <tr>
            <th></th>
            <th>Fecha</th>
            <th>Cantidad</th>
          </tr>
        </thead>
        <tbody>{entries}</tbody>
      </Table>
    </Fetcher>
  );
}

export default TransactionList;
